/*!
* \file joueurList.c
* \brief Fichier contenant le code des fonctions liees aux structures NodeJoueur et ListJoueur.
*/

#include <stdio.h>
#include <stdlib.h>
#include "structures.h"
#include "joueurList.h"

/*!
 * \fn NodeJoueur * initNodeJoueur(Joueur * j)
 * \brief La fonction cree un nouveau NodeJoueur et initialise sa valeur courante.
 *
 * \param La fonction prend en entree un pointeur vers un Joueur.
 * \return NULL si la valeur d'entree est nulle, un pointeur vers le NodeJoueur cree sinon.
 */
NodeJoueur * initNodeJoueur(Joueur * joueur){
    if(joueur!=NULL){
        NodeJoueur * nJ = malloc(sizeof(NodeJoueur));
        nJ->j=joueur;
        nJ->next=NULL;
        return nJ;
    }
    return NULL;
}

/*!
 * \fn void deleteNodeJoueur(NodeJoueur * j)
 * \brief La fonction supprime un NodeJoueur et libere l'espace memoire qui lui etait alloue.
 *
 * \param La fonction prend en entree un pointeur vers un NodeJoueur.
 */
void deleteNodeJoueur(NodeJoueur * j){
    if (j!=NULL){
        free(j);
    }
}

/*!
 * \fn ListJoueur * initJoueurList(ListJoueur * l)
 * \brief La fonction cree et initialise une ListJoueur.
 *
 * \return Un pointeur vers la ListJoueur cree et initialisee.
 */
ListJoueur * initJoueurList(){
    ListJoueur * l = malloc(sizeof(ListJoueur));
    l->firstNodeJoueur=NULL;
    l->currentNodeJoueur=NULL;
    l->lastNodeJoueur=NULL;
    return l;
}

/*!
 * \fn void deleteJoueurList(ListJoueur * l)
 * \brief La fonction supprime une ListJoueur et libere l'espace alloue en memoire.
 *        Attention, il faut egalement liberer les NodeJoueur appartement a la liste.
 * \param Un pointeur vers la ListJoueur a supprimer.
 */
void deleteJoueurList(ListJoueur * l){
    if(l!=NULL){
        free(l);
    }
}

// A DEFINIR, peut etre utile pour l'affichage en console.
//void printJoueurList(ListJoueur * l);

/*!
 * \fn int emptyJoueurList(ListJoueur * l)
 * \brief La fonction teste si la ListJoueur est vide.
 *
 * \param La ListJoueur a tester
 * \return 1 si la liste est vide, 0 sinon
 */
int emptyJoueurList(ListJoueur * l){
    if(l==NULL){
        return 1;
    }
    if(l->firstNodeJoueur==NULL){
        return 1;
    }
    return 0;
}

/*!
 * \fn int outOfJoueurList(ListJoueur * l)
 * \brief La fonction teste si le NodeJoueur courant est la dernier de la liste.
 *
 * \param La ListJoueur a tester
 * \return 1 si le NodeJoueur courant est le dernier de la liste, 0 sinon
 */
int outOfJoueurList(ListJoueur * l){
    if(!emptyJoueurList(l)){
        return (l->currentNodeJoueur==l->lastNodeJoueur);
    }
    return 0;
}

/*!
 * \fn int isFirstJoueurList(ListJoueur * l)
 * \brief La fonction teste si le NodeJoueur courant est le premier de la liste.
 *
 * \param La ListJoueur a tester
 * \return 1 si le NodeJoueur courant est le premier de la liste, 0 sinon
 */
int isFirstJoueurList(ListJoueur * l){
    if(!emptyJoueurList(l)){
        return (l->currentNodeJoueur==l->firstNodeJoueur);
    }
    return 0;
}

/*!
 * \fn int isNextNodeJoueurList(ListJoueur * l, NodeJoueur * n)
 * \brief La fonction teste si le NodeJoueur en parrametre est le NodeJoueur suivant le NodeJoueur courant.
 * \param La ListJoueur et le NodeJoueur a tester.
 * \return 1 si le NodeJoueur en parrametre est le NodeJoueur suivant le NodeJoueur courant, 0 sinon.
 */
int isNextNodeJoueurList(ListJoueur * l, NodeJoueur * n){
    if(!emptyJoueurList(l)){
        return (l->currentNodeJoueur->next==n);
    }
    return 0;
}

/*!
 * \fn void setOnFirstJoueur(ListJoueur * l)
 * \brief La fonction deplace le NodeJoueur courant sur le premier NodeJoueur de la liste.
 *
 * \param La ListJoueur a modifier.
 */
void setOnFirstJoueur(ListJoueur * l){
    if(!emptyJoueurList(l)){
        l->currentNodeJoueur=l->firstNodeJoueur;
    }
}

/*!
 * \fn void nextJoueur(ListJoueur * l)
 * \brief La fonction deplace le NodeJoueur courant sur le NodeJoueur suivant.
 *
 * \param La ListJoueur a modifier.
 */
void nextJoueur(ListJoueur * l){
    if(!emptyJoueurList(l)){
        if(!outOfJoueurList(l)){
            l->currentNodeJoueur=l->currentNodeJoueur->next;
        }
    }
}

/*!
 * \fn Joueur * getCurrentJoueur(ListJoueur * l)
 * \brief La fonction renvoie un pointeur vers le Joueur du NodeJoueur courant.
 *
 * \param La ListJoueur a tester.
 * \return Un pointeur vers le Joueur du noeud courant, NULL si la liste est vide.
 */
Joueur * getCurrentJoueur(ListJoueur * l){
    if(!emptyJoueurList(l)){
        return l->currentNodeJoueur->j;
    }
    return NULL;
}

/*!
 * \fn int addNodeJoueurFirst(ListJoueur * l, NodeJoueur * j)
 * \brief La fonction ajoute un NodeJoueur au debut de la ListJoueur.
 *
 * \param La ListJoueur a modifier et un pointeur vers le NodeJoueur a ajouter.
 * \return 1 si tout s'est bien passe, 0 sinon
 */
int addNodeJoueurFirst(ListJoueur * l, NodeJoueur * j){
    if (j!=NULL){
        if (l!=NULL){
            if (emptyJoueurList(l)){
                l->firstNodeJoueur=j;
                l->currentNodeJoueur=j;
                l->lastNodeJoueur=j;
                return 1;
            }
            else{
                j->next=l->firstNodeJoueur;
                l->firstNodeJoueur=j;
                return 1;
            }
        }
    }
    return 0;
}

/*!
 * \fn int addNodeJoueurLast(ListJoueur * l, NodeJoueur * j)
 * \brief La fonction ajoute un NodeJoueur en fin de la ListJoueur.
 *
 * \param La ListJoueur a modifier et un pointeur vers le NodeJoueur a ajouter.
 * \return 1 si tout s'est bien passe, 0 sinon
 */
int addNodeJoueurLast(ListJoueur * l, NodeJoueur * j){
    if(emptyJoueurList(l)){
        return addNodeJoueurFirst(l,j);
    }
    else if(j!=NULL){
        l->lastNodeJoueur->next=j;
        l->lastNodeJoueur=j;
        return 1;
    }
    return 0;
}

/*!
 * \fn deleteFirstNodeJoueur(ListJoueur * l)
 * \brief La fonction supprime le premier NodeJoueur de la liste et libere l'espace alloue.
 *
 * \param La ListJoueur a modifier.
 * \return 1 si tout s'est bien passe, 0 sinon
 */
int deleteFirstNodeJoueur(ListJoueur * l){
    if(!emptyJoueurList(l)){
        NodeJoueur * tempNodeJoueur = l->firstNodeJoueur;
        l->firstNodeJoueur=l->firstNodeJoueur->next;
        deleteNodeJoueur(tempNodeJoueur);
        return 1;
        }
    return 0;
}

/*!
 * \fn deleteLastNodeJoueur(ListJoueur * l)
 * \brief La fonction supprime le dernier NodeJoueur de la liste et libere l'espace alloue.
 *
 * \param La ListJoueur a modifier.
 * \return 1 si tout s'est bien passe, 0 sinon
 */
int deleteLastNodeJoueur(ListJoueur *l){
    if(!emptyJoueurList(l)){
        if(l->firstNodeJoueur==l->lastNodeJoueur){
            return deleteFirstNodeJoueur(l);
        }
        else{
            NodeJoueur * tempNodeJoueur=l->lastNodeJoueur;
            setOnFirstJoueur(l);
            while(!isNextNodeJoueurList(l,l->lastNodeJoueur)){
                nextJoueur(l);
            }
            l->lastNodeJoueur=l->currentNodeJoueur;
            l->currentNodeJoueur->next=NULL;
            deleteNodeJoueur(tempNodeJoueur);
            return 1;
        }
    }
    return 0;
}

/*!
 * \fn deleteCurrentNodeJoueur(ListJoueur * l)
 * \brief La fonction supprime le NodeJoueur courant de la liste et libre l'espace alloue.
 *
 * \param La ListJoueur a modifier.
 * \return 1 si tout s'est bien passe, 0 sinon
 */
int deleteCurrentNodeJoueur(ListJoueur * l){
    if(isFirstJoueurList(l)){
        return deleteFirstNodeJoueur(l);
    }
    else if(outOfJoueurList(l)){
        return deleteLastNodeJoueur(l);
    }
    else if(!emptyJoueurList(l)){
        NodeJoueur * tempNodeJoueur=l->currentNodeJoueur;
        setOnFirstJoueur(l);
        while(!isNextNodeJoueurList(l,tempNodeJoueur)){
            nextJoueur(l);
        }
        l->currentNodeJoueur->next=tempNodeJoueur->next;
        deleteNodeJoueur(tempNodeJoueur);
        return 1;
    }
    return 0;
}

/*!
 * \fn void setOnJoueur(ListJoueur * l, Joueur * j)
 * \brief La fonction positionne le NodeJoueur courant sur le NodeJoueur possedant la valeur Joueur.
 *
 * \param La ListJoueur a modifier, la valeur de Joueur a trouver.
 */
void setOnJoueur(ListJoueur * l, Joueur * j){
    if(!emptyJoueurList(l)){
        setOnFirstJoueur(l);
        while(j!=getCurrentJoueur(l)){
            nextJoueur(l);
        }
    }
}

/*!
 * \fn int deleteNodeJoueurFromList(ListJoueur * l, Joueur * j)
 * \brief La fonction efface le NodeJoueur contenant le Joueur en parrametre et libere l'espace alloue en memoire.
 *        Elle repositione la valeur courante sur le NodeJoueur courant avant l'appel de la fonction.
 *        Si la valeur a supprimmer est la valeur courante avant l'appel de la fonction, elle positionne la valeur courante sur le NodeJoueur suivant.
 *        Si la valeur a supprimer est le derniere NodeJoueur de la liste, la valeur courante est placee sur le premier NodeJoueur.
 * \param La ListJoueur a modifier, la valeur de Joueur a supprimer.
 * \return 1 si tout c'est bien passe, 0 sinon.
 */
int deleteNodeJoueurFromList(ListJoueur * l, Joueur * j){
    int i;
    if(!emptyJoueurList(l)){
        if(j==getCurrentJoueur(l)){
            i=deleteCurrentNodeJoueur(l);
            nextJoueur(l);
            return i;
        }
        else{
            NodeJoueur * tempCurrent=l->currentNodeJoueur;
            setOnFirstJoueur(l);
            while((getCurrentJoueur(l)!=j)&&(!outOfJoueurList(l))){
                nextJoueur(l);
            }
            if(outOfJoueurList(l)){
                if(getCurrentJoueur(l)!=j){
                    return 0;
                }
                else{
                    i=deleteLastNodeJoueur(l);
                    setOnFirstJoueur(l);
                    return i;
                }
            }
            if(getCurrentJoueur(l)==j){
                i=deleteCurrentNodeJoueur(l);
                l->currentNodeJoueur=tempCurrent;
                return i;
            }
        }
    }
    return 0;
}
